/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001, Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
  [my current email address is on the web servers]

                                  -----
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_priv.h"
#ifdef LCRZODEF_SYSTEM_Linux
 #define _GNU_SOURCE
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <termios.h>
 #include <sys/types.h>
 #include <sys/stat.h>      
#elif defined LCRZODEF_SYSTEM_FreeBSD
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <termios.h>
 #include <sys/types.h>
 #include <sys/stat.h>      
#elif defined LCRZODEF_SYSTEM_Solaris
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <termios.h>
 #include <sys/types.h>
 #include <sys/stat.h>      
#else
 #error "Traiter le cas de LCRZODEF_SYSTEM"
#endif

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_and(lcrzo_etha a, lcrzo_etha b, lcrzo_etha resu)
{ int i;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  if (b==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(a[i]&b[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_or(lcrzo_etha a, lcrzo_etha b, lcrzo_etha resu)
{ int i;
  
  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  if (b==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(a[i]|b[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_xor(lcrzo_etha a, lcrzo_etha b, lcrzo_etha resu)
{ int i;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  if (b==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(a[i]^b[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_not(lcrzo_etha a, lcrzo_etha resu)
{ int i;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(~a[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_inc(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=1;
    for ( i=LCRZO_ETHA_MAXBYTES-1 ; i>=0 ; i-- )
    { if ( a[i]==0xFF && retenue ) /*il va y avoir debordement*/
      { resu[i]=0;
      }
      else
      { resu[i]=(lcrzo_uint8)(a[i]+retenue);
        retenue=0; /*plus de retenue a propager*/
      }
    }
  }
  /*on ignore le debordement de ff:ff:ff:ff:ff:ff*/
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_dec(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=1;
    for ( i=LCRZO_ETHA_MAXBYTES-1 ; i>=0 ; i-- )
    { if ( a[i]==0 && retenue ) /*il va y avoir debordement*/
      { resu[i]=0xFF;
      }
      else
      { resu[i]=(lcrzo_uint8)(a[i]-retenue);
        retenue=0; /*plus de retenue a propager*/
      }
    }
  }
  /*on ignore le debordement de 0:0:0:0:0:0*/
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_shl(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue, r;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=0;
    for ( i=LCRZO_ETHA_MAXBYTES-1 ; i>=0 ; i-- )
    { r=(lcrzo_uint8)(a[i]&0x80);
      resu[i]=(lcrzo_uint8)((a[i]<<1)|retenue);
      retenue=(lcrzo_uint8)(r>>7);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_shr(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue, r;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=0;
    for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { r=(lcrzo_uint8)(a[i]&1);
      resu[i]=(lcrzo_uint8)((a[i]>>1)|(retenue<<7));
      retenue=r;
    }
  }
  return(LCRZO_ERR_OK);
}


/*---------------------------------------------------------------*/
int lcrzo_priv_hdrleth_equal_addr(lcrzo_hdrleth hdrleth,
				  const lcrzo_etha ethasrc,
				  const lcrzo_etha ethadst)
{ return(lcrzo_etha_equal(hdrleth.src, ethasrc) &&
	 lcrzo_etha_equal(hdrleth.dst, ethadst));
}

/*---------------------------------------------------------------*/
int lcrzo_priv_hdrlip_equal_addr(lcrzo_hdrlip hdrlip,
				 const lcrzo_ipl iplsrc,
				 const lcrzo_ipl ipldst)
{ return(lcrzo_ipl_equal(hdrlip.saddr, iplsrc) &&
	 lcrzo_ipl_equal(hdrlip.daddr, ipldst));
}

/*---------------------------------------------------------------*/
int lcrzo_priv_hdrludp_equal_port(lcrzo_hdrludp hdrludp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst)
{ return( (hdrludp.sport==portsrc) && (hdrludp.dport==portdst) );
}

/*---------------------------------------------------------------*/
int lcrzo_priv_hdrltcp_equal_port(lcrzo_hdrltcp hdrltcp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst)
{ return( (hdrltcp.sport==portsrc) && (hdrltcp.dport==portdst) );
}

/*-------------------------------------------------------------*/
int lcrzo_priv_stdin(lcrzo_string *puserstring,
		     lcrzo_int32 *puserstringsize)
{ char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  int c;

  fgetstep=3;
  ptrallocsize=fgetstep;
  lcrzo_er(lcrzo_priv_malloc((lcrzo_data*)&ptr, ptrallocsize));  

  lcrzo_stdin_flush();

  userstringsize=0;
  c=getc(stdin);  
  while ( c!=0x0D && c!=0x0A && c!=0x00 && c!=EOF )
  { ptr[userstringsize]=(char)c;
    userstringsize++;
    c=getc(stdin);
    if (userstringsize==ptrallocsize)
    { ptrallocsize+=fgetstep;
      ptr=realloc(ptr, ptrallocsize);
      if (ptr==NULL) return(LCRZO_ERR_FUREALLOC);
    }
  }
  ptr[userstringsize]='\0';

  if (puserstring!=NULL)
  { *puserstring=ptr;
  }
  else
  { free(ptr);
  }
  if (puserstringsize!=NULL) *puserstringsize=userstringsize;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_passwd(lcrzo_string *puserstring,
		      lcrzo_int32 *puserstringsize)
{ char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  int c;
  struct termios tios;

  fgetstep=3;
  ptrallocsize=fgetstep;
  lcrzo_er(lcrzo_priv_malloc((lcrzo_data*)&ptr, ptrallocsize));  

  lcrzo_stdin_flush();

  if (tcgetattr(STDIN_FILENO, &tios)) return(LCRZO_ERR_FUTCGETATTR);
  tios.c_lflag &= ~ECHO;
  if (tcsetattr(STDIN_FILENO, TCSANOW,  &tios)) return(LCRZO_ERR_FUTCSETATTR);
  userstringsize=0;
  c=getc(stdin);  
  while ( c!=0x0D && c!=0x0A && c!=0x00 && c!=EOF )
  { ptr[userstringsize]=(char)c;
    userstringsize++;
    c=getc(stdin);
    if (userstringsize==ptrallocsize)
    { ptrallocsize+=fgetstep;
      ptr=realloc(ptr, ptrallocsize);
      if (ptr==NULL) return(LCRZO_ERR_FUREALLOC);
    }
  }
  ptr[userstringsize]='\0';
  tios.c_lflag |= ECHO;
  if (tcsetattr(STDIN_FILENO, TCSANOW,  &tios)) return(LCRZO_ERR_FUTCSETATTR);
  puts("");

  if (puserstring!=NULL)
  { *puserstring=ptr;
  }
  else
  { free(ptr);
  }
  if (puserstringsize!=NULL) *puserstringsize=userstringsize;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelempos(lcrzo_int32 elemcount,
			     lcrzo_int32 position,
			     lcrzo_bool wanttoinsertelem,
			     lcrzo_int32 *pposnegative,
			     lcrzo_int32 *ppospositive,
			     int *pdirection, 
			     lcrzo_int32 *pdistance)
{ int sens;
  lcrzo_int32 distance, pospositif, posnegatif;

  /*verification des parametres*/
  lcrzo_bool_verifbof(wanttoinsertelem);

  /*verification et calcul de la position positivee*/
  if (position==0)
  { return(LCRZO_ERR_PONOZERO);
  }
  else if (position>0)
  { if ( position > elemcount+(wanttoinsertelem?1:0) )
    { return(LCRZO_ERR_POTOOBIG);
    }
    posnegatif=position-elemcount-(wanttoinsertelem?2:1);
    pospositif=position;
  }
  else
  { if ( position < -(elemcount+(wanttoinsertelem?1:0)) )
    { return(LCRZO_ERR_POTOOLOW);
    }
    posnegatif=position;
    pospositif=elemcount+(wanttoinsertelem?2:1)+position;
  }

  /*regarde quel sens prendre*/
  if ( pospositif <= -posnegatif )
  { sens=LCRZO_PRIV_INCREASE;
    distance=pospositif;
  }
  else
  { sens=LCRZO_PRIV_DECREASE;
    distance=-posnegatif;
  }

  if (pdirection!=NULL) *pdirection=sens;
  if (pdistance!=NULL) *pdistance=distance;
  if (pposnegative!=NULL) *pposnegative=posnegatif;
  if (ppospositive!=NULL) *ppospositive=pospositif;

  /*  printf("cnt=%ld pos=%ld ins=%d ==> dir %d, distance %ld [%ld,%ld]\n",
	 elemcount, position, wanttoinsertelem,
	 sens, distance, posnegatif, pospositif);*/
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelemrange(lcrzo_int32 elemcount,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance)
{ lcrzo_int32 infpospositive, suppospositive, infsupdistance;
  int infsupdirection;

  lcrzo_er(lcrzo_priv_manageelempos(elemcount, infposition, 0,
				    pinfposnegative, &infpospositive,
				    pinfdirection, pinfdistance));
  lcrzo_er(lcrzo_priv_manageelempos(elemcount, supposition, 0,
				    psupposnegative, &suppospositive,
				    NULL, NULL));

  /*on calcule le nombre d'elements a voir et le sens*/
  if (suppospositive>=infpospositive)
  { infsupdirection=LCRZO_PRIV_INCREASE;
    infsupdistance=suppospositive-infpospositive;
  }
  else
  { infsupdirection=LCRZO_PRIV_DECREASE;
    infsupdistance=infpospositive-suppospositive;
  }

  if (pinfpospositive!=NULL) *pinfpospositive=infpospositive;
  if (psuppospositive!=NULL) *psuppospositive=suppospositive;
  if (pinfsupdirection!=NULL) *pinfsupdirection=infsupdirection;
  if (pinfsupdistance!=NULL) *pinfsupdistance=infsupdistance;

  /*  printf("pos=%ld*%ld[%ld*%ld] ==> dir %d, distance %ld\n",
	 infposition, supposition, infpospositive, suppospositive, 
	 infsupdirection, infsupdistance);
  */
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelemrangefast(lcrzo_int32 elemcount,
				   lcrzo_int32 infposition,
				   lcrzo_int32 supposition,
				   lcrzo_int32 *pinfposnegative,
				   lcrzo_int32 *pinfpospositive,
				   lcrzo_int32 *psupposnegative,
				   lcrzo_int32 *psuppospositive,
				   int *pinfdirection, 
				   lcrzo_int32 *pinfdistance,
				   int *pinfsupdirection, 
				   lcrzo_int32 *pinfsupdistance)
{ lcrzo_int32 infdist, supdist;

  lcrzo_er(lcrzo_priv_manageelempos(elemcount, infposition, 0, NULL, NULL,
				    NULL, &infdist));
  lcrzo_er(lcrzo_priv_manageelempos(elemcount, supposition, 0, NULL, NULL,
				    NULL, &supdist));

  if (infdist<=supdist)
  { lcrzo_er(lcrzo_priv_manageelemrange(elemcount,
					infposition, supposition,
					pinfposnegative, pinfpospositive,
					psupposnegative, psuppospositive,
					pinfdirection, pinfdistance,
					pinfsupdirection, pinfsupdistance));
  }
  else
  { lcrzo_er(lcrzo_priv_manageelemrange(elemcount,
					supposition, infposition,
					pinfposnegative, pinfpospositive,
					psupposnegative, psuppospositive,
					pinfdirection, pinfdistance,
					pinfsupdirection, pinfsupdistance));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_managedataposneg(lcrzo_int32 datasize,
				lcrzo_int32 negative,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive)
{
  if (negative==0) return(LCRZO_ERR_PONOZERO);
  if (negative<datasize-1)return(LCRZO_ERR_POTOOBIG);
  
  if (pnegative!=NULL) *pnegative=negative;
  if (ppositive!=NULL) *ppositive=negative+datasize+1;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_managedatapospos(lcrzo_int32 datasize,
				lcrzo_int32 positive,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive)
{
  if (positive==0) return(LCRZO_ERR_PONOZERO);
  if (positive>datasize+1)return(LCRZO_ERR_POTOOBIG);
  
  if (pnegative!=NULL) *pnegative=positive-datasize-1;
  if (ppositive!=NULL) *ppositive=positive;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_managedatarange(lcrzo_int32 datasize,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance)
{ lcrzo_int32 infposnegative, infpospositive, supposnegative;
  lcrzo_int32 suppospositive, infdistance, infsupdistance;
  int infdirection, infsupdirection;

  if (infposition==0) return(LCRZO_ERR_PONOZERO);
  if (supposition==0) return(LCRZO_ERR_PONOZERO);

  /*check infposition*/
  if (infposition>0)
  { if (infposition>datasize+1)return(LCRZO_ERR_POTOOBIG);
    infposnegative=infposition-datasize-2;
    infpospositive=infposition;
  }
  else
  { if (-infposition>datasize+1)return(LCRZO_ERR_POTOOLOW);
    infposnegative=infposition;
    infpospositive=datasize+2+infposition;
  }

  /*check supposition*/
  if (supposition>0)
  { if (supposition>datasize+1)return(LCRZO_ERR_POTOOBIG);
    supposnegative=supposition-datasize-2;
    suppospositive=supposition;
  }
  else
  { if (-supposition>datasize+1)return(LCRZO_ERR_POTOOLOW);
    supposnegative=supposition;
    suppospositive=datasize+2+supposition;
  }

  /*calculate infdirection and distance*/
  if (infpospositive>(datasize+1)/2)
  { infdirection=LCRZO_PRIV_DECREASE;
    infdistance=-infposnegative-1;
  }
  else
  { infdirection=LCRZO_PRIV_INCREASE;
    infdistance=infpospositive-1;
  }

  /*on calcule le nombre d'elements a voir et le sens*/
  if (suppospositive>=infpospositive)
  { infsupdirection=LCRZO_PRIV_INCREASE;
    infsupdistance=suppospositive-infpospositive;
  }
  else
  { infsupdirection=LCRZO_PRIV_DECREASE;
    infsupdistance=infpospositive-suppospositive;
  }

  if (pinfposnegative!=NULL) *pinfposnegative=infposnegative;
  if (psupposnegative!=NULL) *psupposnegative=supposnegative;
  if (pinfpospositive!=NULL) *pinfpospositive=infpospositive;
  if (psuppospositive!=NULL) *psuppospositive=suppospositive;
  if (pinfdirection!=NULL) *pinfdirection=infdirection;
  if (pinfdistance!=NULL) *pinfdistance=infdistance;
  if (pinfsupdirection!=NULL) *pinfsupdirection=infsupdirection;
  if (pinfsupdistance!=NULL) *pinfsupdistance=infsupdistance;

  /*
  printf("%ld pos=%ld*%ld[%ld*%ld][%ld*%ld] ==> %d_%ld %d_%ld\n", datasize,
	 infposition, supposition, 
	 infposnegative, supposnegative, infpospositive, suppospositive, 
	 infdirection, infdistance, infsupdirection, infsupdistance);
  */
  return(LCRZO_ERR_OK);
}




/*-------------------------------------------------------------*/
/*
void lcrzo_priv_priv_printstat(struct stat st)
{ printf("dev=%ld ino=%ld mode=%d nlink=%d uid=%d gid=%d size=%ld\n",
	 (long)st.st_dev, st.st_ino, st.st_mode, st.st_nlink,
	 st.st_uid, st.st_gid, st.st_size);
}

void lcrzo_priv_uid_print(void)
{ printf("uid=%d euid=%d    gid=%d egid=%d\n", getuid(), geteuid(), getgid(), getegid());
}
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openread_priv(const char *filename,
				int *pfd)
{ int ret, fd;
  struct stat st;
  dev_t dev; ino_t ino; mode_t mode; nlink_t nlink;
  uid_t uid; gid_t gid; off_t size;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);

  /*before openning, we verify if the file seems OK*/
  ret=lstat(filename, &st);
  if (ret==-1) return(LCRZO_ERR_FULSTAT);
  /*lcrzo_priv_priv_printstat(st);*/
  /*determine file type*/
  if (S_ISLNK(st.st_mode))
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "File is a symlink":
		       "Le fichier est un lien symbolique",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }
  if ( !S_ISREG(st.st_mode) )
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "Not a regular file":"Le fichier n'est pas regulier",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }
  /*save for future use*/
  dev=st.st_dev; ino=st.st_ino; mode=st.st_mode; nlink=st.st_nlink;
  uid=st.st_uid; gid=st.st_gid; size=st.st_size;
  
  /*open*/
  fd=open(filename, O_RDONLY);
  if (fd==-1) return(LCRZO_ERR_FUOPEN);

  /*verify if the characteristics are the same*/
  ret=fstat(fd, &st);
  if (ret==-1) { close(fd); return(LCRZO_ERR_FUFSTAT); }
  /*lcrzo_priv_priv_printstat(st);*/
  if ( dev!=st.st_dev || ino!=st.st_ino || mode!=st.st_mode ||
       nlink!=st.st_nlink || uid!=st.st_uid || gid!=st.st_gid ||
       size!=st.st_size )
  { close(fd);
    lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "File has changed":"Le fichier a change",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }

  /*eventually close the file descriptor*/
  if (pfd==NULL)
  { ret=close(fd);
    if ( ret == -1 ) return(LCRZO_ERR_FUCLOSE);
  }
  else *pfd=fd;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openread(const char *filename,
			   int *pfd)
{ int ret, retopen;
  uid_t uid_user, uid_priv;
  gid_t gid_user, gid_priv;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);

  /*save the actual privileges*/
  uid_user=getuid();
  uid_priv=geteuid();
  gid_user=getgid();
  gid_priv=getegid();
  /*lcrzo_priv_uid_print();*/
  /*loose the priv*/
  if (gid_user!=gid_priv)
  { ret=setegid(gid_user);
    if (ret==-1) return(LCRZO_ERR_FUSETEGID);
  }
  if (uid_user!=uid_priv)
  { ret=seteuid(uid_user);
    if (ret==-1) return(LCRZO_ERR_FUSETEUID);
  }
  /*lcrzo_priv_uid_print();*/

  retopen=lcrzo_priv_fd_openread_priv(filename, pfd);

  /*restore the old priv*/
  if (uid_user!=uid_priv)
  { ret=seteuid(uid_priv);
    if (ret==-1) 
    { if (retopen==LCRZO_ERR_OK)
      { if (pfd!=NULL) close(*pfd);
        return(LCRZO_ERR_FUSETEUID);
      }
      else return(retopen);
    }
  }
  if (gid_user!=gid_priv)
  { ret=setegid(gid_priv);
    if (ret==-1)
    { if (retopen==LCRZO_ERR_OK)
      { if (pfd!=NULL) close(*pfd);
        return(LCRZO_ERR_FUSETEGID);
      }
      else return(retopen);
    }
  }
  /*lcrzo_priv_uid_print();*/
  return(retopen);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openwrite_priv(const char *filename,
				 lcrzo_bool mustbeanexistentfile,
				 lcrzo_bool mustbeanewfile,
				 int *pfd)
{ int ret, fd;
  struct stat st;
  dev_t dev; ino_t ino; mode_t mode; nlink_t nlink;
  uid_t uid; gid_t gid; off_t size;
  lcrzo_bool fileexists;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(mustbeanewfile);
  lcrzo_bool_verifbof(mustbeanexistentfile);
  if (mustbeanewfile&&mustbeanexistentfile)
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "Cannot set mustbeanewfile and mustbeanexistentfile":
		      "Ne pas affecter mustbeanewfile et mustbeanexistentfile",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }

  /*before openning, we verify if the file seems OK*/
  ret=lstat(filename, &st);
  if (ret==-1)
  { /*unknown error*/
    if (errno!=ENOENT) return(LCRZO_ERR_FULSTAT);
    errno=0;
    fileexists=0;
    /*only for the compiler*/
    dev=0; ino=0; mode=0; nlink=0; uid=0; gid=0; size=0;
  }
  else
  { fileexists=1;
    /*lcrzo_priv_priv_printstat(st);*/
    /*determine file type*/
    if (S_ISLNK(st.st_mode))
    { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
			 "File is a symlink":
			 "Le fichier est un lien symbolique",
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
      return(LCRZO_ERR_SPFILESECURITY);
    }
    if ( !S_ISREG(st.st_mode) )
    { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
			 "Not a regular file":"Le fichier n'est pas regulier",
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
      return(LCRZO_ERR_SPFILESECURITY);
    }
    /*save for future use*/
    dev=st.st_dev; ino=st.st_ino; mode=st.st_mode; nlink=st.st_nlink;
    uid=st.st_uid; gid=st.st_gid; size=st.st_size;
  }

  /*error cases*/
  if ( mustbeanewfile && fileexists )
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "We want a new file, but file exists":
		       "Nouveau fichier voulu, mais il existe deja",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }
  if ( mustbeanexistentfile && !fileexists )
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "We want an existent file, but file doesn't exists":
		       "Fichier existant voulu, mais il n'existe pas",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }
  
  /*open*/
  if (fileexists)
  { fd=open(filename, O_WRONLY&(~O_CREAT));
    if (fd==-1) return(LCRZO_ERR_FUOPEN);
  }
  else
  { fd=open(filename, O_WRONLY|O_CREAT|O_EXCL, 0600);
    if (fd==-1) return(LCRZO_ERR_FUOPEN);
  }

  /*we have to do that only on existent files*/  
  if (fileexists)
  { /*verify if the characteristics are the same*/
    ret=fstat(fd, &st);
    if (ret==-1) { close(fd); return(LCRZO_ERR_FUFSTAT); }
    /*lcrzo_priv_priv_printstat(st);*/
    if ( dev!=st.st_dev || ino!=st.st_ino || mode!=st.st_mode ||
	 nlink!=st.st_nlink || uid!=st.st_uid || gid!=st.st_gid ||
	 size!=st.st_size )
    { close(fd);
      lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
			 "File has changed":"Le fichier a change",
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
      return(LCRZO_ERR_SPFILESECURITY);
    }
    /*truncate the file*/
    ret=ftruncate(fd, 0);
    if (ret==-1) { close(fd); return(LCRZO_ERR_FUFTRUNCATE); }
  }

  /*eventually close the file descriptor*/
  if (pfd==NULL)
  { ret=close(fd);
    if ( ret == -1 ) return(LCRZO_ERR_FUCLOSE);
  }
  else *pfd=fd;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openwrite(const char *filename,
			  lcrzo_bool mustbeanexistentfile,
			  lcrzo_bool mustbeanewfile,
			  int *pfd)
{ int ret, retopen;
  uid_t uid_user, uid_priv;
  gid_t gid_user, gid_priv;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(mustbeanewfile);
  lcrzo_bool_verifbof(mustbeanexistentfile);
  if (mustbeanewfile&&mustbeanexistentfile)
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "Cannot set mustbeanewfile and mustbeanexistentfile":
		      "Ne pas affecter mustbeanewfile et mustbeanexistentfile",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }

  /*save the actual privileges*/
  uid_user=getuid();
  uid_priv=geteuid();
  gid_user=getgid();
  gid_priv=getegid();
  /*lcrzo_priv_uid_print();*/
  /*loose the priv*/
  if (gid_user!=gid_priv)
  { ret=setegid(gid_user);
    if (ret==-1) return(LCRZO_ERR_FUSETEGID);
  }
  if (uid_user!=uid_priv)
  { ret=seteuid(uid_user);
    if (ret==-1) return(LCRZO_ERR_FUSETEUID);
  }
  /*lcrzo_priv_uid_print();*/

  retopen=lcrzo_priv_fd_openwrite_priv(filename, mustbeanexistentfile,
				     mustbeanewfile, pfd);

  /*restore the old priv*/
  if (uid_user!=uid_priv)
  { ret=seteuid(uid_priv);
    if (ret==-1) 
    { if (retopen==LCRZO_ERR_OK)
      { if (pfd==NULL) close(*pfd); 
        return(LCRZO_ERR_FUSETEUID);
      }
      else return(retopen);
    }
  }
  if (gid_user!=gid_priv)
  { ret=setegid(gid_priv);
    if (ret==-1)
    { if (retopen==LCRZO_ERR_OK)
      { if (pfd==NULL) close(*pfd);
        return(LCRZO_ERR_FUSETEGID);
      }
      else return(retopen);
    }
  }
  /*lcrzo_priv_uid_print();*/
  return(retopen);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openappend_priv(const char *filename,
				lcrzo_bool mustbeanexistentfile,
				int *pfd)
{ int ret, fd;
  struct stat st;
  dev_t dev; ino_t ino; mode_t mode; nlink_t nlink;
  uid_t uid; gid_t gid; off_t size;
  lcrzo_bool fileexists;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(mustbeanexistentfile);

  /*before openning, we verify if the file seems OK*/
  ret=lstat(filename, &st);
  if (ret==-1)
  { /*unknown error*/
    if (errno!=ENOENT) return(LCRZO_ERR_FULSTAT);
    errno=0;
    fileexists=0;
    /*only for the compiler*/
    dev=0; ino=0; mode=0; nlink=0; uid=0; gid=0; size=0;
  }
  else
  { fileexists=1;
    /*lcrzo_priv_priv_printstat(st);*/
    /*determine file type*/
    if (S_ISLNK(st.st_mode))
    { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
			 "File is a symlink":
			 "Le fichier est un lien symbolique",
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
      return(LCRZO_ERR_SPFILESECURITY);
    }
    if ( !S_ISREG(st.st_mode) )
    { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
			 "Not a regular file":"Le fichier n'est pas regulier",
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
      return(LCRZO_ERR_SPFILESECURITY);
    }
    /*save for future use*/
    dev=st.st_dev; ino=st.st_ino; mode=st.st_mode; nlink=st.st_nlink;
    uid=st.st_uid; gid=st.st_gid; size=st.st_size;
  }

  /*error cases*/
  if ( mustbeanexistentfile && !fileexists )
  { lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
		       "We want an existent file, but file doesn't exists":
		       "Fichier existant voulu, mais il n'existe pas",
		       LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
    return(LCRZO_ERR_SPFILESECURITY);
  }
  
  /*open*/
  if (fileexists)
  { fd=open(filename, O_WRONLY|O_APPEND);
    if (fd==-1) return(LCRZO_ERR_FUOPEN);
  }
  else
  { fd=open(filename, O_WRONLY|O_CREAT|O_EXCL, 0600);
    if (fd==-1) return(LCRZO_ERR_FUOPEN);
  }

  /*we have to do that only on existent files*/  
  if (fileexists)
  { /*verify if the characteristics are the same*/
    ret=fstat(fd, &st);
    if (ret==-1) { close(fd); return(LCRZO_ERR_FUFSTAT); }
    /*lcrzo_priv_priv_printstat(st);*/
    if ( dev!=st.st_dev || ino!=st.st_ino || mode!=st.st_mode ||
	 nlink!=st.st_nlink || uid!=st.st_uid || gid!=st.st_gid ||
	 size!=st.st_size )
    { close(fd);
      lcrzo_string_init_text((lcrzo_global.language==LCRZO_GLOBAL_FRLANG)?
			 "File has changed":"Le fichier a change",
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);
      return(LCRZO_ERR_SPFILESECURITY);
    }
  }

  /*eventually close the file descriptor*/
  if (pfd==NULL)
  { ret=close(fd);
    if ( ret == -1 ) return(LCRZO_ERR_FUCLOSE);
  }
  else *pfd=fd;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openappend(const char *filename,
			   lcrzo_bool mustbeanexistentfile,
			   int *pfd)
{ int ret, retopen;
  uid_t uid_user, uid_priv;
  gid_t gid_user, gid_priv;

  /*parameters verification*/
  if (filename==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(mustbeanexistentfile);

  /*save the actual privileges*/
  uid_user=getuid();
  uid_priv=geteuid();
  gid_user=getgid();
  gid_priv=getegid();
  /*lcrzo_priv_uid_print();*/
  /*loose the priv*/
  if (gid_user!=gid_priv)
  { ret=setegid(gid_user);
    if (ret==-1) return(LCRZO_ERR_FUSETEGID);
  }
  if (uid_user!=uid_priv)
  { ret=seteuid(uid_user);
    if (ret==-1) return(LCRZO_ERR_FUSETEUID);
  }
  /*lcrzo_priv_uid_print();*/

  retopen=lcrzo_priv_fd_openappend_priv(filename, mustbeanexistentfile, pfd);

  /*restore the old priv*/
  if (uid_user!=uid_priv)
  { ret=seteuid(uid_priv);
    if (ret==-1) 
    { if (retopen==LCRZO_ERR_OK)
      { if (pfd==NULL) close(*pfd);
        return(LCRZO_ERR_FUSETEUID);
      }
      else return(retopen);
    }
  }
  if (gid_user!=gid_priv)
  { ret=setegid(gid_priv);
    if (ret==-1)
    { if (retopen==LCRZO_ERR_OK)
      { if (pfd==NULL) close(*pfd);
        return(LCRZO_ERR_FUSETEGID);
      }
      else return(retopen);
    }
  }
  /*lcrzo_priv_uid_print();*/
  return(retopen);
}


/*---------------------------------------------------------------*/
int lcrzo_priv_fd_block(int fd, lcrzo_bool etrebloquant)
{ int old_fc, new_fc, retour;

  /*verification des parametres*/
  lcrzo_bool_verifbof(etrebloquant);

  old_fc=fcntl(fd, F_GETFL, 0);
  if (old_fc<0) return(LCRZO_ERR_FUFCNTL);
  if (etrebloquant)
    new_fc = old_fc & (~O_NONBLOCK);
  else
    new_fc = old_fc | O_NONBLOCK;
  retour=fcntl(fd, F_SETFL, new_fc);
  if (retour<0) return(LCRZO_ERR_FUFCNTL);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_write(int fd,
			lcrzo_constdata donnees,
			lcrzo_int32 nboctdonnees)
{ int retour;
  
  /*parameters verification*/
  if (donnees==NULL && nboctdonnees) return(LCRZO_ERR_SPNULLPTRSIZE);

  if (donnees==NULL || nboctdonnees==0) return(LCRZO_ERR_OK);

  retour=write(fd, donnees, nboctdonnees);
  if (retour==-1) return(LCRZO_ERR_FUWRITE);
  if (retour!=(int)nboctdonnees)
  { if (lcrzo_global.language==LCRZO_GLOBAL_FRLANG)
    { lcrzo_string_init_text("la fonction write() a ecrit seulement ",
			 LCRZO_GLOERRMSG_MAXBYTES,
			 lcrzo_global.errmsglcrzo);
      lcrzo_string_append_int(retour, "%d", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
      lcrzo_string_append_text(" octets au lieu d'ecrire ",
			   LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
      lcrzo_string_append_int(nboctdonnees, "%d", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
      lcrzo_string_append_text(" octets.", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
    }
    else
    { lcrzo_string_init_text("write() wrote only ",
			 LCRZO_GLOERRMSG_MAXBYTES,
			 lcrzo_global.errmsglcrzo);
      lcrzo_string_append_int(retour, "%d", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
      lcrzo_string_append_text(" bytes over ", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
      lcrzo_string_append_int(nboctdonnees, "%d", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
      lcrzo_string_append_text(" bytes.", LCRZO_GLOERRMSG_MAXBYTES,
			   lcrzo_global.errmsglcrzo);
    }
    return(LCRZO_ERR_FUWRITE);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_readfixed(int fd,
			    lcrzo_bool etrebloquant,
			    lcrzo_int32 nboctdonneesalire,
			    lcrzo_data dataout)
{ lcrzo_int32 nboct, nbocttotal;
  lcrzo_uint8 array[100];
  
  /*verification des parametres*/
  lcrzo_bool_verifbof(etrebloquant);

  nbocttotal=0;
  while(1)
  { if (dataout==NULL)
    { nboct=read(fd, array, 
		 ((nboctdonneesalire-nbocttotal)>99)?
		 99:(nboctdonneesalire-nbocttotal));
    }
    else
    { nboct=read(fd, dataout+nbocttotal, nboctdonneesalire-nbocttotal);
    }
    /*une erreur s'est produite*/
    if (nboct==-1)
    { if (errno!=EAGAIN)
      { /*c'etait une erreur fatale*/
	return(LCRZO_ERR_FUREAD);
      }
      else
      { /*annule l'erreur, c'est simplement que rien n'etait disponible*/
	errno=0;
	if (!etrebloquant)
        { /*si on a trouve quelque chose, on doit quand meme continuer*/
	  if (!nbocttotal)
	  { return(LCRZO_ERR_OKTEMPDATAEND);
	  }
        }
      }
    }
    /*on a lu quelque chose*/
    else
    { nbocttotal+=nboct;
      /*on a lu ce que l'on voulait*/
      if ( nbocttotal==nboctdonneesalire )
      { return(LCRZO_ERR_OK);
      }
    }
  }
  /*on ne doit jamais atteindre ce return*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_readfixedm(int fd,
			     lcrzo_bool etrebloquant,
			     lcrzo_int32 nboctdonneesalire,
			     lcrzo_data *pdataout)
{ lcrzo_int32 nboct, nbocttotal;
  lcrzo_data ptr;
  
  /*verification des parametres*/
  lcrzo_bool_verifbof(etrebloquant);

  lcrzo_er(lcrzo_priv_malloc(&ptr, nboctdonneesalire));

  nbocttotal=0;
  while(1)
  { nboct=read(fd, ptr+nbocttotal, nboctdonneesalire-nbocttotal);
    /*une erreur s'est produite*/
    if (nboct==-1)
    { if (errno!=EAGAIN)
      { /*c'etait une erreur fatale*/
	free(ptr);
        return(LCRZO_ERR_FUREAD);
      }
      else
      { /*annule l'erreur, c'est simplement que rien n'etait disponible*/
	errno=0;
	if (!etrebloquant)
        { /*si on a trouve quelque chose, on doit quand meme continuer*/
	  if (!nbocttotal)
	  { free(ptr);
            return(LCRZO_ERR_OKTEMPDATAEND);
	  }
        }
      }
    }
    /*on a lu quelque chose*/
    else
    { nbocttotal+=nboct;
      /*on a lu ce que l'on voulait*/
      if ( nbocttotal==nboctdonneesalire )
      { if (pdataout!=NULL) *pdataout=ptr;
        return(LCRZO_ERR_OK);
      }
    }
  }
  /*on ne doit jamais atteindre ce return*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_read(int fd,
		       lcrzo_bool etrebloquant,
		       lcrzo_int32 dataoutmaxsize,
		       lcrzo_data dataout,
		       lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 nboct;
  lcrzo_uint8 array[100];
  
  /*verification des parametres*/
  lcrzo_bool_verifbof(etrebloquant);

  while(1)
  { if (dataout==NULL)
    { nboct=read(fd, array,(dataoutmaxsize>99)?99:dataoutmaxsize);
    }
    else
    { nboct=read(fd, dataout, dataoutmaxsize);
    }
    /*une erreur s'est produite*/
    if (nboct==-1)
    { if (errno!=EAGAIN)
      { /*c'etait une erreur fatale*/
	return(LCRZO_ERR_FUREAD);
      }
      else
      { /*annule l'erreur, c'est simplement que rien n'etait disponible*/
	errno=0;
	if (!etrebloquant)
        { if (pdataoutsize!=NULL) *pdataoutsize=0;
	  return(LCRZO_ERR_OKTEMPDATAEND);
        }
      }
    }
    /*on a lu quelque chose*/
    else
    { /*on a lu ce que l'on voulait*/
      if (pdataoutsize!=NULL) *pdataoutsize=nboct;
      return(LCRZO_ERR_OK);
    }
  }
  /*on ne doit jamais atteindre ce return*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_readm(int fd,
			lcrzo_bool etrebloquant,
			lcrzo_data *pdataout,
			lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 nboct;
  lcrzo_data ptr;
  
  /*verification des parametres*/
  lcrzo_bool_verifbof(etrebloquant);

  /*we read maxi 66000 bytes*/
  lcrzo_er(lcrzo_priv_malloc(&ptr, 66000));

  while(1)
  { nboct=read(fd, ptr, 65999);
    /*une erreur s'est produite*/
    if (nboct==-1)
    { if (errno!=EAGAIN)
      { /*c'etait une erreur fatale*/
	free(ptr);
        return(LCRZO_ERR_FUREAD);
      }
      else
      { /*annule l'erreur, c'est simplement que rien n'etait disponible*/
	errno=0;
	if (!etrebloquant)
        { if (pdataoutsize!=NULL) *pdataoutsize=0;
	  free(ptr);
	  return(LCRZO_ERR_OKTEMPDATAEND);
        }
      }
    }
    /*on a lu quelque chose*/
    else
    { lcrzo_er(lcrzo_priv_realloc(&ptr, nboct));
      if (pdataout!=NULL) *pdataout=ptr;
      if (pdataoutsize!=NULL) *pdataoutsize=nboct;
      return(LCRZO_ERR_OK);
    }
  }
  /*on ne doit jamais atteindre ce return*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_malloc(lcrzo_data *ptr,
		      lcrzo_int32 allocsize)
{
  /*parameters verification*/
  if (ptr==NULL) return(LCRZO_ERR_IEINTERNALERROR);
  if (allocsize<0) return(LCRZO_ERR_PATOOLOW);

  /*"+1" because we use it to put the '\0' of a string in lcrzo_string_xxx
      functions.*/
  *ptr=malloc(allocsize+1);
  if (*ptr==NULL) return(LCRZO_ERR_FUMALLOC);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_realloc(lcrzo_data *ptr,
		       lcrzo_int32 newallocsize)
{
  /*parameters verification*/
  if (ptr==NULL) return(LCRZO_ERR_IEINTERNALERROR);
  if (newallocsize<0) return(LCRZO_ERR_PATOOLOW);

  /*"+1" because realloc(p,0)==free(p), so to protect against double free
      Moreover, we use it to put the '\0' of a string in lcrzo_string_xxx
      functions.*/
  if (*ptr==NULL)
  { *ptr=malloc(newallocsize+1);
    if (*ptr==NULL) return(LCRZO_ERR_FUMALLOC);
  }
  else
  { *ptr=realloc(*ptr, newallocsize+1);
    if (*ptr==NULL) return(LCRZO_ERR_FUREALLOC);
  }
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_data(lcrzo_constdata datain,
				  lcrzo_int32 datainsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_data(datain, datainsize, *pstringoutsize,
				stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_data(lcrzo_constdata datain,
				   lcrzo_int32 datainsize,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_data(datain, datainsize, *pstringoutsize,
				 NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_range(lcrzo_conststring stringin,
				   lcrzo_int32 stringinstartpos,
				   lcrzo_int32 stringinendpos,
				   lcrzo_int32 stringoutmaxsize,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_range((lcrzo_constdata)stringin, strlen(stringin),
				 stringinstartpos, stringinendpos,
				 *pstringoutsize,
				 stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_range(lcrzo_conststring stringin,
				    lcrzo_int32 stringinstartpos,
				    lcrzo_int32 stringinendpos,
				    lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_range((lcrzo_constdata)stringin, strlen(stringin),
				  stringinstartpos, stringinendpos,
				  *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_fill(lcrzo_conststring stringin,
				  lcrzo_int32 fillingsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_fill((lcrzo_constdata)stringin, strlen(stringin), fillingsize,
				*pstringoutsize,
				stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_fill(lcrzo_conststring stringin,
				   lcrzo_int32 fillingsize,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_fill((lcrzo_constdata)stringin, strlen(stringin), fillingsize,
				 *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_char(lcrzo_uint8 c,
				  lcrzo_int32 fillingsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_char(c, fillingsize,
				*pstringoutsize,
				stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_char(lcrzo_uint8 c,
				   lcrzo_int32 fillingsize,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_char(c, fillingsize,
				 *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_rand(lcrzo_uint8 minvalue,
				  lcrzo_uint8 maxvalue,
				  lcrzo_int32 fillingsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_rand(minvalue, maxvalue, fillingsize,
				*pstringoutsize,
				stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_rand(lcrzo_uint8 minvalue,
				   lcrzo_uint8 maxvalue,
				   lcrzo_int32 fillingsize,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_rand(minvalue, maxvalue, fillingsize,
				 *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_int(lcrzo_int32 integer,
				 const char * format,
				 lcrzo_int32 stringoutmaxsize,
				 lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_int(integer, format,
			       *pstringoutsize,
			       stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_int(lcrzo_int32 integer,
				  const char * format,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_int(integer, format,
				*pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_text(const char * text,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_text(text,
				*pstringoutsize,
				stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_text(const char * text,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_text(text,
				 *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_coretext(const char * text,
				      lcrzo_int32 stringoutmaxsize,
				      lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_coretext(text,
				    *pstringoutsize,
				    stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_coretext(const char * text,
				       lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_coretext(text,
				     *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_hexa(const char * hexa,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_hexa(hexa,
				*pstringoutsize,
				stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_hexa(const char * hexa,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_hexa(hexa,
				 *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_mixed(const char * mixed,
				   lcrzo_int32 stringoutmaxsize,
				   lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_mixed(mixed,
				 *pstringoutsize,
				 stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_mixed(const char * mixed,
				    lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_mixed(mixed,
				  *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_base64(const char * base64,
				    lcrzo_int32 stringoutmaxsize,
				    lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_base64(base64,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_base64(const char * base64,
				     lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_base64(base64,
				   *pstringoutsize, NULL, pstringoutsize));
}


/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_text_stdin(const char * message,
					const char * defaulttext,
					lcrzo_int32 stringoutmaxsize,
					lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_text_stdin(message, defaulttext,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_text_stdin(const char * message,
					 const char * defaulttext,
					 lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_text_stdin(message, defaulttext,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_ct_stdin(const char * message,
				      const char * defaultcoretext,
				      lcrzo_int32 stringoutmaxsize,
				      lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_coretext_stdin(message, defaultcoretext,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_ct_stdin(const char * message,
				       const char * defaultcoretext,
				       lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_coretext_stdin(message, defaultcoretext,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_text_passwd(const char * message,
					 const char * defaulttext,
					 lcrzo_int32 stringoutmaxsize,
					 lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_text_passwd(message, defaulttext,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_text_passwd(const char * message,
					  const char * defaulttext,
					  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_text_passwd(message, defaulttext,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_ct_passwd(const char * message,
				       const char * defaultcoretext,
				       lcrzo_int32 stringoutmaxsize,
				       lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_coretext_passwd(message, defaultcoretext,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_ct_passwd(const char * message,
					const char * defaultcoretext,
					lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_coretext_passwd(message, defaultcoretext,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_hexa_stdin(const char * message,
					const char * defaulthexa,
					lcrzo_int32 stringoutmaxsize,
					lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_hexa_stdin(message, defaulthexa,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_hexa_stdin(const char * message,
					 const char * defaulthexa,
					 lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_hexa_stdin(message, defaulthexa,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_mixed_stdin(const char * message,
					 const char * defaultmixed,
					 lcrzo_int32 stringoutmaxsize,
					 lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_mixed_stdin(message, defaultmixed,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_mixed_stdin(const char * message,
					  const char * defaultmixed,
					  lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_mixed_stdin(message, defaultmixed,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_b64_stdin(const char * message,
				       const char * defaultbase64,
				       lcrzo_int32 stringoutmaxsize,
				       lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_append_base64_stdin(message, defaultbase64,
				  *pstringoutsize,
				  stringoutmaxsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_string_appendm_b64_stdin(const char * message,
					const char * defaultbase64,
					lcrzo_int32 *pstringoutsize)
{ if (pstringoutsize==NULL) return(LCRZO_ERR_PANULLPTR);
  return(lcrzo_data_appendm_base64_stdin(message, defaultbase64,
				   *pstringoutsize, NULL, pstringoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_init_data(lcrzo_constdata datain,
				  lcrzo_uint16 datainsize,
				  lcrzo_uint16 dataoutmaxsize,
				  lcrzo_data dataout,
				  lcrzo_uint16 *pdataoutsize)
{ return(lcrzo_priv_data_u16_append_data(datain, datainsize, 0,
				    dataoutmaxsize, dataout, pdataoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_initm_data(lcrzo_constdata datain,
				   lcrzo_uint16 datainsize,
				   lcrzo_data *pdataout,
				   lcrzo_uint16 *pdataoutsize)
{ lcrzo_data ptr;

  ptr=NULL;
  lcrzo_er(lcrzo_priv_data_u16_appendm_data(datain, datainsize, 0,
				       &ptr, pdataoutsize));
  if (pdataout!=NULL) *pdataout=ptr;
  else free(ptr);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_append_data(lcrzo_constdata datain,
				    lcrzo_uint16 datainsize,
				    lcrzo_uint16 dataoutcurrentsize,
				    lcrzo_uint16 dataoutmaxsize,
				    lcrzo_data dataout,
				    lcrzo_uint16 *pdataoutsize)
{
  /*parameters verification*/
  if (datain==NULL && datainsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (dataoutcurrentsize>dataoutmaxsize) return(LCRZO_ERR_PADATACURSUPMAX);

  if (dataoutcurrentsize+datainsize<=dataoutmaxsize)
  { if (dataout!=NULL)
    { memcpy(dataout+dataoutcurrentsize, datain, datainsize);
    }
    if (pdataoutsize!=NULL) 
    { *pdataoutsize=(lcrzo_uint16)(dataoutcurrentsize+datainsize);
    }
    return(LCRZO_ERR_OK);
  }
  else
  { if (dataout!=NULL)
    { memcpy(dataout+dataoutcurrentsize, datain,
             dataoutmaxsize-dataoutcurrentsize);
    }
    if (pdataoutsize!=NULL) *pdataoutsize=dataoutmaxsize;
    return(LCRZO_ERR_OKDATATRUNCATED);
  }
 
  /*should never be here*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_appendm_data(lcrzo_constdata datain,
				     lcrzo_uint16 datainsize,
				     lcrzo_uint16 dataoutcurrentsize,
				     lcrzo_data *pdataout,
				     lcrzo_uint16 *pdataoutsize)
{ lcrzo_uint16 dataoutsize;

  /*parameters verification*/
  if (datain==NULL && datainsize) return(LCRZO_ERR_SPNULLPTRSIZE);

  /*compute the output size*/
  dataoutsize=(lcrzo_uint16)(dataoutcurrentsize+datainsize);
  if (pdataoutsize!=NULL) 
  { *pdataoutsize=(lcrzo_uint16)dataoutsize;
  }

  /*init pdataout*/
  if (pdataout!=NULL)
  { lcrzo_er(lcrzo_priv_realloc(pdataout, dataoutsize));
    memcpy(*pdataout+dataoutcurrentsize, datain, datainsize);
  }

  return(LCRZO_ERR_OK);
}
