
/*us    Laurent Constantin's network library (lcrzo)
                 lcrzo_priv module

  Functions herein are private.
*/
/*fr    Bibliotheque reseau de Laurent Constantin (lcrzo)
                 Module lcrzo_priv

  Les fonctions presentes dans ce module sont privees a la bibliotheque.
*/


#include "lcrzo.h"

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_PRINTTYPE_COUNT    35
#define LCRZO_PRIV_PRINTPROFILE_COUNT  8

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_and(lcrzo_etha a, lcrzo_etha b, lcrzo_etha result);
int lcrzo_priv_etha_or(lcrzo_etha a, lcrzo_etha b, lcrzo_etha result);
int lcrzo_priv_etha_xor(lcrzo_etha a, lcrzo_etha b, lcrzo_etha result);
int lcrzo_priv_etha_not(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_inc(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_dec(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_shl(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_shr(lcrzo_etha a, lcrzo_etha result);

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrleth_equal_addr(lcrzo_hdrleth hdrleth,
				  const lcrzo_etha ethasrc,
				  const lcrzo_etha ethadst);
int lcrzo_priv_hdrlip_equal_addr(lcrzo_hdrlip hdrlip,
				 const lcrzo_ipl iplsrc,
				 const lcrzo_ipl ipldst);
int lcrzo_priv_hdrludp_equal_port(lcrzo_hdrludp hdrludp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst);
int lcrzo_priv_hdrltcp_equal_port(lcrzo_hdrltcp hdrltcp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst);

/*-------------------------------------------------------------*/
int lcrzo_priv_stdin(lcrzo_string *puserstring,
		     lcrzo_int32 *puserstringsize);
int lcrzo_priv_passwd(lcrzo_string *puserstring,
		      lcrzo_int32 *puserstringsize);


/*---------------------------------------------------------------*/
#define LCRZO_PRIV_INCREASE 1
#define LCRZO_PRIV_DECREASE 2

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelempos(lcrzo_int32 elemcount,
			     lcrzo_int32 position,
			     lcrzo_bool wanttoinsertelem,
			     lcrzo_int32 *pposnegative,
			     lcrzo_int32 *ppospositive,
			     int *pdirection, 
			     lcrzo_int32 *pdistance);
int lcrzo_priv_manageelemrange(lcrzo_int32 elemcount,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance);
int lcrzo_priv_manageelemrangefast(lcrzo_int32 elemcount,
				   lcrzo_int32 infposition,
				   lcrzo_int32 supposition,
				   lcrzo_int32 *pinfposnegative,
				   lcrzo_int32 *pinfpospositive,
				   lcrzo_int32 *psupposnegative,
				   lcrzo_int32 *psuppospositive,
				   int *pinfdirection, 
				   lcrzo_int32 *pinfdistance,
				   int *pinfsupdirection, 
				   lcrzo_int32 *pinfsupdistance);

/*-------------------------------------------------------------*/
int lcrzo_priv_managedataposneg(lcrzo_int32 datasize,
				lcrzo_int32 negative,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive);
int lcrzo_priv_managedatapospos(lcrzo_int32 datasize,
				lcrzo_int32 positive,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive);
int lcrzo_priv_managedatarange(lcrzo_int32 datasize,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance);

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_openread_priv(const char *filename,
				int *pfd);
int lcrzo_priv_fd_openread(const char *filename,
			   int *pfd);
int lcrzo_priv_fd_openwrite_priv(const char *filename,
				 lcrzo_bool mustbeanexistentfile,
				 lcrzo_bool mustbeanewfile,
				 int *pfd);
int lcrzo_priv_fd_openwrite(const char *filename,
			    lcrzo_bool mustbeanexistentfile,
			    lcrzo_bool mustbeanewfile,
			    int *pfd);
int lcrzo_priv_fd_openappend_priv(const char *filename,
				  lcrzo_bool mustbeanexistentfile,
				  int *pfd);
int lcrzo_priv_fd_openappend(const char *filename,
			     lcrzo_bool mustbeanexistentfile,
			     int *pfd);

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_block(int fd, 
			lcrzo_bool beblocking);

int lcrzo_priv_fd_write(int fd, 
			lcrzo_constdata data,
			lcrzo_int32 nboctdata);

int lcrzo_priv_fd_readfixed(int fd,
			    lcrzo_bool beblocking,
			    lcrzo_int32 datasizetoread,
			    lcrzo_data dataout);
int lcrzo_priv_fd_readfixedm(int fd,
			     lcrzo_bool beblocking,
			     lcrzo_int32 datasizetoread,
			     lcrzo_data *pdataout);
int lcrzo_priv_fd_read(int fd,
		       lcrzo_bool beblocking,
		       lcrzo_int32 dataoutmaxsize,
		       lcrzo_data dataout,
		       lcrzo_int32 *pdataoutsize);
int lcrzo_priv_fd_readm(int fd,
			lcrzo_bool beblocking,
			lcrzo_data *pdataout,
			lcrzo_int32 *pdataoutsize);


/*-------------------------------------------------------------*/
int lcrzo_priv_malloc(lcrzo_data *ptr,
		      lcrzo_int32 allocsize);
int lcrzo_priv_realloc(lcrzo_data *ptr,
		       lcrzo_int32 newallocsize);

/*-------------------------------------------------------------*/
int lcrzo_priv_string_append_data(lcrzo_constdata datain,
				  lcrzo_int32 datainsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_data(lcrzo_constdata datain,
				   lcrzo_int32 datainsize,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_range(lcrzo_conststring stringin,
				   lcrzo_int32 stringinstartpos,
				   lcrzo_int32 stringinendpos,
				   lcrzo_int32 stringoutmaxsize,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_range(lcrzo_conststring stringin,
				    lcrzo_int32 stringinstartpos,
				    lcrzo_int32 stringinendpos,
				    lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_fill(lcrzo_conststring stringin,
				  lcrzo_int32 fillingsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_fill(lcrzo_conststring stringin,
				   lcrzo_int32 fillingsize,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_char(lcrzo_uint8 c,
				  lcrzo_int32 fillingsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_char(lcrzo_uint8 c,
				   lcrzo_int32 fillingsize,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_rand(lcrzo_uint8 minvalue,
				  lcrzo_uint8 maxvalue,
				  lcrzo_int32 fillingsize,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_rand(lcrzo_uint8 minvalue,
				   lcrzo_uint8 maxvalue,
				   lcrzo_int32 fillingsize,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_int(lcrzo_int32 integer,
				 const char * format,
				 lcrzo_int32 stringoutmaxsize,
				 lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_int(lcrzo_int32 integer,
				  const char * format,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_text(const char * text,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_text(const char * text,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_coretext(const char * text,
				      lcrzo_int32 stringoutmaxsize,
				      lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_coretext(const char * text,
				       lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_hexa(const char * hexa,
				  lcrzo_int32 stringoutmaxsize,
				  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_hexa(const char * hexa,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_mixed(const char * mixed,
				   lcrzo_int32 stringoutmaxsize,
				   lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_mixed(const char * mixed,
				    lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_base64(const char * base64,
				    lcrzo_int32 stringoutmaxsize,
				    lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_base64(const char * base64,
				     lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_text_stdin(const char * message,
					const char * defaulttext,
					lcrzo_int32 stringoutmaxsize,
					lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_text_stdin(const char * message,
					 const char * defaulttext,
					 lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_ct_stdin(const char * message,
				      const char * defaultcoretext,
				      lcrzo_int32 stringoutmaxsize,
				      lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_ct_stdin(const char * message,
				       const char * defaultcoretext,
				       lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_text_passwd(const char * message,
					 const char * defaulttext,
					 lcrzo_int32 stringoutmaxsize,
					 lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_text_passwd(const char * message,
					  const char * defaulttext,
					  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_ct_passwd(const char * message,
				       const char * defaultcoretext,
				       lcrzo_int32 stringoutmaxsize,
				       lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_ct_passwd(const char * message,
					const char * defaultcoretext,
					lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_hexa_stdin(const char * message,
					const char * defaulthexa,
					lcrzo_int32 stringoutmaxsize,
					lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_hexa_stdin(const char * message,
					 const char * defaulthexa,
					 lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_mixed_stdin(const char * message,
					 const char * defaultmixed,
					 lcrzo_int32 stringoutmaxsize,
					 lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_mixed_stdin(const char * message,
					  const char * defaultmixed,
					  lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_append_b64_stdin(const char * message,
				       const char * defaultbase64,
				       lcrzo_int32 stringoutmaxsize,
				       lcrzo_int32 *pstringoutsize);
int lcrzo_priv_string_appendm_b64_stdin(const char * message,
					const char * defaultbase64,
					lcrzo_int32 *pstringoutsize);

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_init_data(lcrzo_constdata datain,
				  lcrzo_uint16 datainsize,
				  lcrzo_uint16 dataoutmaxsize,
				  lcrzo_data dataout,
				  lcrzo_uint16 *pdataoutsize);
int lcrzo_priv_data_u16_initm_data(lcrzo_constdata datain,
				   lcrzo_uint16 datainsize,
				   lcrzo_data *pdataout,
				   lcrzo_uint16 *pdataoutsize);
int lcrzo_priv_data_u16_append_data(lcrzo_constdata datain,
				    lcrzo_uint16 datainsize,
				    lcrzo_uint16 dataoutcurrentsize,
				    lcrzo_uint16 dataoutmaxsize,
				    lcrzo_data dataout,
				    lcrzo_uint16 *pdataoutsize);
int lcrzo_priv_data_u16_appendm_data(lcrzo_constdata datain,
				     lcrzo_uint16 datainsize,
				     lcrzo_uint16 dataoutcurrentsize,
				     lcrzo_data *pdataout,
				     lcrzo_uint16 *pdataoutsize);

/*-------------------------------------------------------------*/
/*us system dependant functions */
/*fr fonctions dependant du systeme */
#include "lcrzo_sysdep.h"
